

// Test all range variants.

#include <stdio.h>
#include <string.h>

#include "ad++.h"


const int n_cases = 24 ;

const int n = 50 ;

Range sample(char* desc, Dimension d, int i_case) {
  Range x ;

  switch(i_case) {
    case 0 :
      x = Range(n) ;
      strcpy(desc, "Range(n)") ;
      break ;
    case 1 :
      x = BlockRange(n, d) ;
      strcpy(desc, "BlockRange(n, d)") ;
      break ;
    case 2 :
      x = CyclicRange(n, d) ;
      strcpy(desc, "CyclicRange(n, d)") ;
      break ;
    case 3 :
      x = StepRange(n, d) ;
      strcpy(desc, "StepRange(n, d)") ;
      break ;
    case 4 :
      x = BlockRange(n, d, 12) ;
      strcpy(desc, "BlockRange(n, d, 12)") ;
      break ;
    case 5 : {
      x = BlockCyclicRange(n, d, 4) ;
      strcpy(desc, "BlockCyclicRange(n, d, 4)") ;
      break ;
    }
    case 6 :
      x = Range(400).subrng(n, 100, 5) ;
      strcpy(desc, "Range(400).subrng(n, 100, 5)") ;
      break ;
    case 7 :
      x = Range(250).subrng(n, 225, -4) ;
      strcpy(desc, "Range(250).subrng(n, 225, -4)") ;
      break ;
    case 8 :
      x = BlockRange(400, d).subrng(n, 100, 5) ;
      strcpy(desc, "BlockRange(400, d).subrng(n, 100, 5)") ;
      break ;
    case 9 :
      //x = BlockRange(400, d, LAY_PACKED).subrng(n, 100, 5) ;
      //strcpy(desc, "BlockRange(400, d, LAY_PACKED).subrng(n, 100, 5)") ;
      x = ADAPTBlockRange(n, 100, 5, 400, d, 67) ;
      strcpy(desc, "ADAPTBlockRange(n, 100, 5, 400, d, 67)") ;
      break ;
    case 10 :
      x = BlockRange(250, d).subrng(n, 225, -4) ;
      strcpy(desc, "BlockRange(250, d).subrng(n, 225, -4)") ;
      break ;
    case 11 :
      //x = BlockRange(250, d, LAY_PACKED).subrng(n, 225, -4) ;
      //strcpy(desc, "BlockRange(250, d, LAY_PACKED).subrng(n, 225, -4)") ;
      x = ADAPTBlockRange(n, 225, -4, 250, d, 42) ;
      strcpy(desc, "ADAPTBlockRange(n, 225, -4, 250, d, 42)") ;
      break ;
    case 12 :
      x = CyclicRange(400, d).subrng(n, 100, 5) ;
      strcpy(desc, "CyclicRange(400, d).subrng(n, 100, 5)") ;
      break ;
    case 13 :
      //x = CyclicRange(400, d, LAY_ADAPT).subrng(n, 100, 5) ;
      //strcpy(desc, "CyclicRange(400, d, LAY_ADAPT).subrng(n, 100, 5)") ;
      x = ADAPTCyclicRange(n, 100, 5, d) ;
      strcpy(desc, "ADAPTCyclicRange(n, 100, 5, d)") ;
      break ;
    case 14 :
      x = CyclicRange(250, d).subrng(n, 225, -4) ;
      strcpy(desc, "CyclicRange(250, d).subrng(n, 225, -4)") ;
      break ;
    case 15 :
      //x = CyclicRange(250, d, LAY_ADAPT).subrng(n, 225, -4) ;
      //strcpy(desc, "CyclicRange(250, d, LAY_ADAPT).subrng(n, 225, -4)") ;
      x = ADAPTCyclicRange(n, 225, -4, d) ;
      strcpy(desc, "ADAPTCyclicRange(n, 225, -4, d)") ;
      break ;
    case 16 :
      x = StepRange(400, d).subrng(n, 100, 5) ;
      strcpy(desc, "StepRange(400, d).subrng(n, 100, 5)") ;
      break ;
    case 17 :
      x = StepRange(250, d).subrng(n, 225, -4) ;
      strcpy(desc, "StepRange(250, d).subrng(n, 225, -4)") ;
      break ;
    case 18 :
      x = BlockRange(400, d, 120).subrng(n, 100, 5) ;
      strcpy(desc, "BlockRange(400, d, 120).subrng(n, 100, 5)") ;
      break ;
    case 19 :
      x = BlockRange(250, d, 60).subrng(n, 225, -4) ;
      strcpy(desc, "BlockRange(250, d, 60).subrng(n, 225, -4)") ;
      break ;
    case 20 : {
      x = BlockCyclicRange(400, d, 24).subrng(n, 100, 5) ;
      strcpy(desc, "BlockCyclicRange(400, d, 24).subrng(n, 100, 5)") ;
      break ;
    }
    case 21 : {
      x = BlockCyclicRange(250, d, 17).subrng(n, 225, -4) ;
      strcpy(desc, "BlockCyclicRange(250, d, 17).subrng(n, 225, -4)") ;
      break ;
    }
    case 22 : {
      //x = BlockCyclicRange(400, d, 24, LAY_PACKED).subrng(n, 100, 5) ;
      //strcpy(desc,
      //       "BlockCyclicRange(400, d, 24, LAY_PACKED).subrng(n, 100, 5)") ;
      x = ADAPTBlockCyclicRange(n, 100, 5, d, 24) ;
      strcpy(desc, "ADAPTBlockCyclicRange(n, 100, 5, d, 24)") ;
      break ;
    }
    case 23 : {
      //x = BlockCyclicRange(250, d, 17, LAY_PACKED).subrng(n, 225, -4) ;
      //strcpy(desc,
      //       "BlockCyclicRange(250, d, 17, LAY_PACKED).subrng(n, 225, -4)") ;
      x = ADAPTBlockCyclicRange(n, 225, -4, d, 17) ;
      strcpy(desc, "ADAPTBlockCyclicRange(n, 225, -4, d, 17)") ;
      break ;
    }
  }
  return x ;
}

int main(int argc, char* argv[]) {
  AdlibInit(argc, argv) ;

  char xDesc [1024], yDesc [1024] ;

  Procs1 p(6) ;
  Dimension d = p.dim(0) ;

  for(int i = 0 ; i < n_cases ; i++) {
    Range x = sample(xDesc, d, i) ;
    for(int j = 0 ; j < n_cases ; j++) {
      Range y = sample(yDesc, d, j) ;

      gprintf("x = %s\n", xDesc) ;
      gprintf("y = %s\n", yDesc) ;

      Array1<float> a(x, p) ;
      Array1<float> b(y, p) ;
      Array1<int> mask1(y, p), mask2(y, p) ;

      ON(p) {
        Index i(x) ;
	OVERALL(i) {
	  a(i) = i ;
	} ALLOVER(i) ;
      } NO(p) ;

      remap(b, a) ;

      ON(p) {
        Index i(y) ;
	OVERALL(i) {
	  mask1(i) = b(i) == i ;
	  mask2(i) = b(i) != i ;
	} ALLOVER(i) ;
      } NO(p) ;

      if(!all(mask1) || any(mask2))
	    // Do both checks to reduce chance of bug in reductions.
        aprintf("%2R0%f\n", b) ;
      else
	gprintf("Checked OK\n") ;

    //aprintf("%2R0%4d\n", b) ;
    }
  }

  AdlibFinalize() ;
}


